/*
 *	$Id: Thread.m,v 1.1 2004/05/19 05:00:20 rh Exp $
 *
 * Implementation of a very simple Thread object in Objective C
 *
 */

#include <errno.h>
#import "Thread.h"

/*
 * Rather than attempting to do some dirty tricks trying to get
 * C to invoke Objective-C methods (which would be not portable),
 * we just invoke the Objective-C run method from a child helper
 * function.
 */

static void *child(void *object)
{
	id thread = object;

	return [thread run];	/* invoke run method */
}


/*
 * Now for the Objective-C thread class implementation
 */

@implementation Thread

/*
 * The run method does nothing by default.  It needs to be
 * overridden by a subclass of Thread to do something useful
 */
- run
{
	return self;
}

/*
 * The start method first checks if a thread has already been
 * created -- if so, it does nothing and returns nil, otherwise
 * it attempts to create a new thread and invoke the runWithParams
 * method on it.
 */
- startWithParams: (void *) passed_params
{
	if (tid)			// thread already running?
	{
		errno = EEXIST;
		return nil;
	}

	params = passed_params;		// store thread parameters

	if (pthread_create(&tid, NULL, child, self) != 0)
		return nil;		// thread creation failed

	return self;
}

/*
 * simple start method without parameters
 */
- start
{
	return [self startWithParams: NULL];
}

/*
 * Wait for the thread to have finished
 */
- joinWithReturnValue: (void **) retval
{
	return pthread_join(tid, retval) ? nil : self;
}

/*
 * simple join method without return value
 */
- join
{
	return [self joinWithReturnValue: NULL];
}

/*
 * get the startup parameters
 */
- getParams
{
	return params;
}

@end
